//////////////////////////////////////////////////////////////////////////////////////
/*
Cellular Texture generation

Author: Henrik Krysell
E-mail: drx@drx.dk
Website: www.drx.dk

Copyright (C) 2004  Henrik Krysell

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//////////////////////////////////////////////////////////////////////////////////////

#ifndef _CELLULAR_H__
#define _CELLULAR_H__

/////////////////// INCLUDE & DEFINE ///////////////////
#include <string.h>
#include "Vector3.h"
#include "TextureManager.h"

/////////////////// CLASS - Cellular ///////////////////
namespace Math3D
{

using Manager::TextureMan;
using Manager::UINT;

enum eCelTypes {CEL_SQUAREDIST,CEL_INV_SQUAREDIST,CEL_DIST2_MINUS_DIST1, CEL_DIST1_MUL_DIST2,CEL_INV_DIST2_MINUS_DIST1, CEL_INV_DIST1_MUL_DIST2};

class Cellular
{
public:
	// FUNCTIONS
	Cellular(int points, int width, int height, Vector3 color, int type);
	Cellular(const Cellular &cel);
	Cellular();
	~Cellular();
	
	char *getData() { return pMap; };
	int getWidth() { return width; };
	int getHeight() { return height; };
	UINT getTexture() { return texture; };
	
	Cellular &operator= (const Cellular &cel);
	Cellular operator+ (Cellular cel);
	Cellular operator- (Cellular cel);
	
	bool make(int points, int width, int height, Vector3 color, int type);
	UINT createTexture();
	UINT createTexture(int points, int width, int height, Vector3 color, int type);
		
	// VARIABLES

private:
	// FUNCTIONS
	Vector3 getDistance(Vector2 pos);
	float wrapDist(Vector2 pos, Vector2 point);
	
	// VARIABLES
	int 			width;
	int 			height;
	int 			numPoints;
	UINT		 	texture;
	Vector2 		*pPoints;
	Vector3			color;
	char 			*pMap;
	int		 		type;
};

} // namespace
#endif
