//////////////////////////////////////////////////////////////////////////////////////
/*
Author: Henrik Krysell
E-mail: drx@drx.dk
Website: www.drx.dk

Copyright (C) 2004  Henrik Krysell

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//////////////////////////////////////////////////////////////////////////////////////

#ifndef _SINGLETON_H
#define _SINGLETON_H

// INCLUDE

namespace Singleton
{

template <class T> class Phoenix
{
public:
	static T &instance();
	
private:
	// FUNCTIONS
	virtual ~Phoenix() { destroyed = true; pInstance = NULL; };
	
	static void create();
	static void onDeadReference();
	static void killPhoenix();
	
	// VARIABLES
	static T *pInstance;
	static bool destroyed;
	
}; // Phoenix



template <class T>T *Phoenix<T>::pInstance = 0;
template <class T> bool Phoenix<T>::destroyed = false;

template <class T> T &Phoenix<T>::instance()
{
	if (!pInstance)
	{
		if (destroyed)
			onDeadReference();
		else
			create();
	}
	return *pInstance;
}

///////////////////////////////////////////////////////////////////////////////
template <class T> void Phoenix<T>::create()
{
	static T theInstance;
	pInstance = &theInstance;
}

template <class T> void Phoenix<T>::onDeadReference()
{
	// get the "shell" of the destroyed singleton
	create();
	// from the ashes you shall be reborn!
	new(pInstance) T;
	atexit(killPhoenix);
	destroyed = false;
}

template <class T> void Phoenix<T>::killPhoenix()
{
	destroyed = true;
	pInstance = NULL;
}

} // namespace

#endif

