//////////////////////////////////////////////////////////////////////////////////////
/*
Author: Henrik Krysell
E-mail: drx@drx.dk
Website: www.drx.dk

Copyright (C) 2004  Henrik Krysell

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//////////////////////////////////////////////////////////////////////////////////////

#ifndef _TEXTURE_MANAGER_H
#define _TEXTURE_MANAGER_H

// INCLUDE
#include <iostream>
#include <cstring>
#include <GL/gl.h>
#include <GL/glu.h>
#include <cstdlib>

#include "Singleton.h"

// DEFINE

#define GL_CLAMP_TO_EDGE		0x812F						// This is for our skybox textures
#define NUM_TEXTURES		100							// starting size of the texture array

#define TGA_RGB		 2		// This tells us it's a normal RGB (really BGR) file
#define TGA_A		 3		// This tells us it's a ALPHA file
#define TGA_RLE		10		// This tells us that the targa is Run-Length Encoded (RLE)

namespace Manager
{

// create singleton type
class TextureManager;
typedef Singleton::Phoenix<TextureManager> TextureMan;

#ifdef __linux__
	typedef unsigned long UINT;
	typedef unsigned short WORD;
	typedef unsigned char byte;
#endif


struct sTexture
{
	UINT ID;
	char filename[64];
};

// This is our image structure for our targa data
struct tImage
{
	int channels;			// The channels in the image (3 = RGB : 4 = RGBA)
	int sizeX;				// The width of the image in pixels
	int sizeY;				// The height of the image in pixels
	unsigned char *data;	// The image pixel data
};

// CLASSES
class TextureManager {
 friend class Singleton::Phoenix<TextureManager>;
 
public:

// PUBLIC - Functions
	bool loadTexture(UINT &textureid, char *strFileName, bool clamp);
	bool createDataTexture(UINT &texture, char *pImageBits, int width, int height);
	void createLightmapTexture(UINT &texture, char *pImageBits, int width, int height);
	
	void printStatus(char *param);		// prints Texture status to console

	void destroy();

// PUBLIC - Variables

private:

// PRIVATE - Functions
	// prevent multiple instances when using singletons
	TextureManager();
	~TextureManager();
	TextureManager(const TextureManager &o) { };
	TextureManager operator=(TextureManager o) { return *this; };
	
	bool createTexture(UINT &texture, const char *strFileName, bool clamp);
	void increaseArray();
	tImage *loadTGA(const char *strFileName);

// PRIVATE - Variables

	sTexture *textures;
	int numTextures;
	int maxTextures;
};

} // namespace
#endif
