//////////////////////////////////////////////////////////////////////////////////////
/*
Author: Henrik Krysell
E-mail: drx@drx.dk
Website: www.drx.dk

Copyright (C) 2004  Henrik Krysell

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//////////////////////////////////////////////////////////////////////////////////////

#ifndef _CVECTOR3_H
#define _CVECTOR3_H

//#include "Const.h"
//#include <windows.h>
#include <math.h>

namespace Math3D
{

/////////////////// CLASS - Vector2 ///////////////////

class Vector2 
{
public:

		// A default constructor
	Vector2() {}

	// This is our constructor that allows us to initialize our data upon creating an instance
	Vector2(float X, float Y) 
	{ 
		x = X; y = Y;
	}

	// Here we overload the + operator so we can add vectors together 
	Vector2 operator+(Vector2 vVector)
	{
		// Return the added vectors result.
		return Vector2(vVector.x + x, vVector.y + y);
	}

	// Here we overload the - operator so we can subtract vectors 
	Vector2 operator-(Vector2 vVector)
	{
		// Return the subtracted vectors result
		return Vector2(x - vVector.x, y - vVector.y);
	}
	
	// Here we overload the * operator so we can multiply by scalars
	Vector2 operator*(float num)
	{
		// Return the scaled vector
		return Vector2(x * num, y * num);
	}

	// Here we overload the / operator so we can divide by a scalar
	Vector2 operator/(float num)
	{
		// Return the scale vector
		return Vector2(x / num, y / num);
	}

	float x, y;
};

// This is our basic 3D point/vector class
class Vector3
{
public:
	
	// A default constructor
	Vector3() {}

	// This is our constructor that allows us to initialize our data upon creating an instance
	Vector3(float X, float Y, float Z) 
	{ 
		x = X; y = Y; z = Z;
	}

	// Here we overload the + operator so we can add vectors together 
	Vector3 operator+(Vector3 vVector)
	{
		// Return the added vectors result.
		return Vector3(vVector.x + x, vVector.y + y, vVector.z + z);
	}
	Vector3 operator+(float add)
	{
		// Return the added vectors result.
		return Vector3(x + add,y + add,z + add);
	}

	// Here we overload the - operator so we can subtract vectors 
	Vector3 operator-(Vector3 vVector)
	{
		// Return the subtracted vectors result
		return Vector3(x - vVector.x, y - vVector.y, z - vVector.z);
	}
	Vector3 operator-(float min)
	{
		// Return the added vectors result.
		return Vector3(x - min,y - min,z - min);
	}
	// Here we overload the - operator so we can subtract vectors 
	Vector3 operator-()
	{
		// Return the subtracted vectors result
		return Vector3(-x,-y,-z);
	}
	
	// Here we overload the * operator so we can multiply by scalars
	Vector3 operator*(float num)
	{
		// Return the scaled vector
		return Vector3(x * num, y * num, z * num);
	}

	// Here we overload the / operator so we can divide by a scalar
	Vector3 operator/(float num)
	{
		// Return the scale vector
		return Vector3(x / num, y / num, z / num);
	}

	bool operator==(Vector3 v)
	{
		if ((x == v.x) && (y == v.y) && (z == v.z))
			return true;

		return false;
	}
	void Copy(Vector3 vVector)
	{
		x = vVector.x;
		y = vVector.y;
		z = vVector.z;
	}

	void Abs()
	{
		if (x < 0)
			x = -x;
		if (y < 0)
			y = -y;
		if (z < 0)
			z = -z;
	}
	Vector3 elemMul(Vector3 v) { return Vector3(x*v.x,y*v.y,z*v.z); };

	Vector3 Cross(Vector3 vVector2);
	float Magnitude();
	void Normalize();
	float Dot(Vector3 v){return ( (x * v.x) + (y * v.y) + (z * v.z) );}

	float x, y, z;
};

} // namespace
#endif
