//////////////////////////////////////////////////////////////////////////////////////
/*
Cellular texture

Author: Henrik Krysell
E-mail: drx@drx.dk
Website: www.drx.dk

Copyright (C) 2004  Henrik Krysell

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//////////////////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <string.h>

#include "OglSdl.h"
#include "SDL.h"                                           // The SDL Header File
#include "ErrorHandler.h"
#include "TextureManager.h"
#include "Vector3.h"
#include "CellularTxt.h"

using Manager::TextureMan;
#ifdef __linux__
using Manager::UINT;
#endif

//////////////// GLOBAL VARIABLES ////////////////
OglSdl::OglSdl *win;
UINT texture;

//////////////////////////////////////////////////////////////////////////////////////

void render()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
    
	//     Position      View     Up Vector
	gluLookAt(0, 0, 6,     0, 0, 0,     0, 1, 0);
	
	win->orthoModeFullScreen();
	
	glBindTexture(GL_TEXTURE_2D,texture);
	glBegin (GL_QUADS);
		glTexCoord2f(0,0);
		glVertex2f(0, 0);
		glTexCoord2f(0,600/300);
		glVertex2f(0, 600);  
		glTexCoord2f(800/400,600/300);
		glVertex2f(800, 600); 
		glTexCoord2f(800/400,0);
		glVertex2f(800, 0); 
	glEnd();
	
	win->perspectiveMode();
    
    SDL_GL_SwapBuffers();
}

void mainLoop(void)
{
	bool done = false;
	SDL_Event event;

	while(! done)
	{
		while( SDL_PollEvent(& event) )
		{
			switch ( event.type ) 
			{
			case SDL_QUIT:
				done = true;
				break;
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym)
				{
				case SDLK_ESCAPE:
					done = true;
				break;
				}				
			break;
			case SDL_VIDEORESIZE:
				win->resizeWindow(event.resize.w, event.resize.h);
			break;

			default:
			break;
			} // switch
		} // while( SDL_ ...

		// start rendering
		render();
	} // while( ! done)
}

void init()
{
	glEnable(GL_DEPTH_TEST); 
	glEnable(GL_TEXTURE_2D);

/*
	// Hide the mouse
	if ( SDL_ShowCursor(SDL_DISABLE) != SDL_DISABLE )
	cerr << SDL_GetError() << endl;
*/
	// Enable Key Repeat
	if( SDL_EnableKeyRepeat(100,SDL_DEFAULT_REPEAT_INTERVAL) )
	{
		throw ErrHand::Error("SDL_EnableKeyRepeat FAILED!",2);
	}
	
	Math3D::Cellular celr, celg, celb, cel;
	celr.make(100,400,300,Math3D::Vector3(1,0,0),Math3D::CEL_DIST2_MINUS_DIST1);
	celg.make(200,400,300,Math3D::Vector3(0,1,0),Math3D::CEL_SQUAREDIST);
	celb.make(300,400,300,Math3D::Vector3(0,0,1),Math3D::CEL_DIST2_MINUS_DIST1);
	cel = celr + celg + celb;
	texture = cel.createTexture();
	
	std::cout << "ok Done!\n";
}

int main()
{
	try
	{
		win = new OglSdl::OglSdl;
		win->createWindow("Cellular texture",800,600,24);
		init();
		mainLoop();
	}
	catch (ErrHand::Error err)
	{
		err.printError();
	}
	
	delete win;
	return 0;
}
